#epmi pip
#pip install pyserial

import subprocess
import time
import serial
import argparse
from argparse import RawTextHelpFormatter
import datetime
import threading

POLL_SLEEP_TIME = 0.1 #Задержка между проверками наличия запроса от ардуины (cек)
RECONNECT_SLEEP_TIME = 1 #Задержка между попытками открыть serial порт при потере связи (cек)
TIMEOUT_TIME = 3 #Считается что ардуина перестала отвечать, если от нее не было запросов в течение этого времени (cек)

PORT:str = None
SPEED:int = None
temp = '50'
connected = False
input_thread_running = False




def temp_input():
    global temp
    while True:
        temp_input = input('\033[60C' + 'Enter up to 3 temperature values (comma-separated): ')
        
        error = False
        comma_pos = 0
        find_start_index = 0
        #парсим введенную строку:
        for i in range(3):
            comma_pos = temp_input.find(',', find_start_index) #поиск запятой

            if comma_pos != -1 and i == 2: #если есть третья запятая
                print("Can't be more than 3 values")
                error = True
                break
            
            #выделяем число
            if comma_pos == -1: #если запятой не найдено, то до конца строки
                temp_substring = temp_input[find_start_index:]
            else:
                temp_substring = temp_input[find_start_index:comma_pos]

            try: #проверка, введено ли число
                int(temp_substring)
            except ValueError:
                print('Temperature value must be a number')
                error = True
                break
            
            if comma_pos == -1: #если запятых больше нет, выходим из цикла
                break

            find_start_index = comma_pos + 1

        if not error: #если всё хорошо, введенную строку можно отправлять
            temp = temp_input



parser = argparse.ArgumentParser(
    description = 'This script sends PC disk temperature via a serial port',
    formatter_class=RawTextHelpFormatter
)

parser.add_argument('-m', '--manual',
    required = False,
    action='store_true',
    help = 'send temperature from user input instead of hddtemp',
)

parser.add_argument('device',
    type = str,
    nargs = '?',
    help = 'serial device to commuicate with. Example: /dev/ttyUSB0',
    metavar = 'SERIAL'
)
parser.add_argument('speed',
    type = int,
    nargs = '?',
    help = 'serial port baud rate',
    metavar = 'SPEED'
)

args = parser.parse_args()


#проверка наличия аргументов
if args.device == None:
    print('Serial device was not specified')
    exit()
if args.speed == None:
    print('Serial port baud rate was not specified')
    exit()
if args.manual:
    input_thread = threading.Thread(target = temp_input)
    temp = '50,51,52'

PORT = args.device
SPEED = args.speed

try: #пытаемся открыть serial порт
    serial_port = serial.Serial(
        port = PORT,
        baudrate = SPEED,
        parity = serial.PARITY_NONE,
        stopbits=serial.STOPBITS_ONE,
    )
except OSError: 
    print('An error has occured while opening the serial port')
    exit()

print('Connecting to the serial device...')
serial_port.reset_input_buffer()
while True:
    try:
        if serial_port.in_waiting > 0:
            if not connected:
                if serial_port.readline().decode()[:-2] == 'start_transmission':
                    last_serial_input_time = datetime.datetime.now().timestamp()
                    serial_port.write('starting_transmission\n'.encode())
                    print('Connected. Starting transmission')
                    connected = True
                    if args.manual and not input_thread_running:
                        input_thread.start()
                        input_thread_running = True
            
            incoming_line = serial_port.readline().decode()[:-2]

            if 'req_temperature' in incoming_line:
                last_serial_input_time = datetime.datetime.now().timestamp()

                if args.manual:
                    serial_port.write(('temperature_' + temp + '\n').encode()) #отправка значения
                else:
                    output = str(subprocess.Popen(['netcat', 'localhost', '7634'], stdout = subprocess.PIPE, stderr = subprocess.STDOUT).communicate()[0]) #получаем output от hddtemp
                    #b'|/dev/sda2|ST340014A|41|C|'
                    if output == "b''":
                        print('Hddtemp output is empty, make sure it is running. If not, launch it with #hddtemp -d')
                        exit()
                    
                    #вычленяем значение температуры:
                    temp_pos_end = output.find('|C|')
                    temp_pos_start = output.rfind('|', 0, temp_pos_end) + 1
                    temp = output[temp_pos_start:temp_pos_end]
                    try: 
                        int(temp)
                        #print(temp + ' C')
                        serial_port.write(('temperature_' + temp + '\n').encode()) #отправка значения
                    except ValueError:
                        print('Error: Failed to get a temperature value from hddtemp output')

            if 'send_rpm' in incoming_line:
                send_rpm_pos = incoming_line.find('send_rpm_')
                rpm_string = incoming_line[send_rpm_pos + 9:]
                rpm_string = rpm_string.replace('_', ' ')
                rpm_string = 'Fan RPMs: ' + rpm_string
                #сохранение положения курсора, перервод в начало строки, перевод на 59 символов вправо, очистка всего перед курсором, курсор в начало строки, вывод оборотов, загрузка положения строки
                print('\0337' + '\033[255D' +'\033[59C' + '\033[1K' + '\033[255D' + rpm_string + '\0338', end = '', flush=True)

    except OSError:
        if connected:
            print('\nSerial divice was disconnected. Trying to reconnect...')
            connected = False
        
        try: #пытаемся снова открыть serial порт
            serial_port = serial.Serial(
                port = '/dev/' + PORT,
                baudrate = SPEED
            )
            connected = True
        except OSError: print('...')

        time.sleep(RECONNECT_SLEEP_TIME)
        
    if connected and datetime.datetime.now().timestamp() - last_serial_input_time > TIMEOUT_TIME:
        print('\nSerial device is not responding')
        connected = False

    time.sleep(POLL_SLEEP_TIME)